@@ -2,7 +2,7 @@ from abc import ABCMeta, abstractmethod
 import inspect
 from pathlib import Path
 import sys
-from typing import List, Optional, Sequence
+from typing import List, Optional, Sequence, Mapping
 import uuid
 
 import mincepy
@@ -21,7 +21,7 @@ class Command(mincepy.SimpleSavable, metaclass=ABCMeta):
         self._args = mincepy.RefList(args)
 
     @property
-    def args(self) -> Sequence:
+    def args(self) -> tuple:
         return self._args
 
     @abstractmethod
@@ -50,10 +50,10 @@ def command(cmd, args: Sequence = (), type: str = 'python-function', **kwargs) -
 
 class PythonCommand(Command):
     TYPE_ID = uuid.UUID('61736206-729b-4a0b-9fac-6b5e71123ba0')
-    ATTRS = ('_script_file', '_function', '_kwargs')
+    ATTRS = ('_script_file', '_function', '_kwargs', '_dynamic')
 
     @classmethod
-    def build(cls, cmd, args: Sequence = (), dynamic=False, **kwargs):
+    def build(cls, cmd, args: Sequence = (), dynamic=False, kwargs: dict = None, **rest):
         function = 'run'  # The default function name
 
         if inspect.ismethod(cmd):
@@ -73,7 +73,7 @@ class PythonCommand(Command):
         else:
             raise ValueError("Unknown python function command '{}".format(cmd))
 
-        return PythonCommand(script_file, function, args, dynamic=dynamic, **kwargs)
+        return PythonCommand(script_file, function, args, dynamic=dynamic, kwargs=kwargs, **rest)
 
     # pylint: disable=too-many-arguments
     def __init__(self,
@@ -96,8 +96,7 @@ class PythonCommand(Command):
         """
         if dynamic:
             # Create the arguments for run_dynamically
-            kwargs = dict(script_file=script_file, function=function, args=args, kwargs=kwargs)
-            args = ()  # Just rely on the kwargs
+            args = ((script_file, function),) + args
             script_file = pyshim.__file__
             function = pyshim.run_dynamically.__name__
 
@@ -108,12 +107,17 @@ class PythonCommand(Command):
         self._script_file = self._historian.create_file(script_file.name, 'utf-8')
         self._script_file.from_disk(script_file)
 
+        self._dynamic = dynamic
         self._function = function
-        self._kwargs = kwargs or {}
+        self._kwargs = mincepy.RefDict(kwargs or {})
 
     def __str__(self):
         return "{}@{}{}".format(self._script_file.filename, self._function, self._args)
 
+    @property
+    def dynamic(self) -> bool:
+        return self._dynamic
+
     @property
     def script_file(self):
         """Access the python script file"""
@@ -124,6 +128,10 @@ class PythonCommand(Command):
         """The name of the function that will be run in the script"""
         return self._function
 
+    @property
+    def kwargs(self) -> Mapping:
+        return self._kwargs
+
     def run(self) -> Optional[List]:
         """Run this python command"""
         with self._script_file.open() as file:
